//************************************************************************
// ATmega644 zu VDIP1 ber SPI
//************************************************************************


#include <avr/io.h>
#include <string.h>									// Stringfunktion
#include <stdio.h>									// Standartfunktion
#include <stdlib.h>
//#include <conio.h>
//#include <delay.h>								// Pausenfuktion
#define F_CPU 1000000UL
#include <util/delay.h>
//************************************************************************
// Definebereich VDIP1
//************************************************************************
//*


#define R_SCLK    PORTB &= ~(1<<PIN0) 	//P1_0 		// SCLK 	Port 1 Pin 0 	Taktsignal 			fr VDIP1
#define S_SCLK    PORTB |=  (1<<PIN0)

#define R_RE_VDIP PORTB &= ~(1<<PIN1)  	//P1_1		// RE_VNC 	Port 1 Pin 1 	Hardware Reset 		fr VDIP1
#define S_RE_VDIP PORTB |=  (1<<PIN1)

#define R_CS_VDIP PORTB &= ~(1<<PIN2)  	//P1_2		// CS_VNC 	Port 1 Pin 2 	Chip Select Input 	vom VDIP1
#define S_CS_VDIP PORTB |=  (1<<PIN2)

#define R_SDI     PORTB &= ~(1<<PIN3) 	//P1_3		// SDI 	 	Port 1 Pin 3 	Serial Daten Input 	vom VDIP1
#define S_SDI     PORTB |=  (1<<PIN3)

#define SDO_VDIP   PINB &= ( 1<<PIN4) 	//P1_4		// SDO	 	Port 1 Pin 4 	Serial Daten Output vom VDIP1
		
//************************************************************************
// Statische Variablen
//************************************************************************
static char 			vdip1_check 	= 2;		// 1 = Prfung Programmier-Port und Fehlermeldungen
													// 2 = Prfung nur Programmier-Port
													// verwendet in main zum aufruf "vdip1_prfe_anwesenheit"

static char vdip1_Programmier_Port_ok 	= 1;		// verwendet in main zum berprfen des zustandes des VDIP1 Moduls
													// verwendet in "vdip1_bin_file_bescheiben"

static char				vdip1_ch		= 1;		// VDIP1_CECK 
													// verwendet in "vdip1_bin_file_bescheiben" Zustandsabfrage

static char				file_neu		= 0;		// Neue Datei erzeugen 
static char 			file_erweitern 	= 1;		// Inhalt an Datei anhngen
													// verwendet in main bei Datenablage VDIP1 Modul 
													// neue Datei erzeugen oder in Datei ablegen


//************************************************************************
// Global vernderbare Variablen
//************************************************************************
volatile char lese_buffer[10];       				// Empfangsbuffer (10Byte gro)
													// verwendet in der Funktion "vdip1_byte_lesen"

volatile char sende_buffer[20];      				// Sendebuffer    (20Byte gro)


volatile char vdip1_status;               			// Status Bit fr die Empfangs-Daten
													// verwendet in der Funktion "vdip1_byte_lesen"

//************************************************************************
// Binrer Datenstz fr VDIP1
//************************************************************************
char *file1[] 	 = {"Test0",						// Inhalt Test-File 1 (Text)
					"Test1",
					"Test2",
					"\x0FF"}; 						// File-Endekennung

char file2[] 	 = {0xFA,0x89,0x11,0x23,0xAA,0x02,0x00,0x1F,	
	  				0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
  					0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
  					0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
  					0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
  					0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
  					0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
  					0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
  					0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
  					0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC};


//************************************************************************
// Funktionen Deklaration
//************************************************************************
void vdip1_init(void);								// VDIP1 Initialisierung
void vdip1_byte_lesen(char);						// aus VDIP1 ein Byte lesen
char vdip1_pruefen(char);								// VDIP1 Prfen
char vdip1_byte_schreiben(char);					// VDIP1 Byte schreiben
void vdip1_cmd_schreiben(char*);					// VDIP1 Befehl schreiben
void vdip1_bin_file_bescheiben(char*, char*, int); 	// in VDIP1 ein Binre Datei beschreiben
void vdip1_txtfile_schreiben(char*, char*, char); 	// in VDIP1 eine Textdatei schreiben

//************************************************************************
// Hauptprogramm
//************************************************************************
int main (void) 
	{
_delay_ms(2000);

		vdip1_init();     							// VNC1L-Initialisierung
		DDRA  = 0xFF;		
		DDRB  = 0x0F;
		DDRC  = 0xFF;								// PortB die oberen 4 Bit als eingang die unteren 4 Bit als Ausgang
		PORTC = 0xFF;
//***************************
  // Hauptschleife
//***************************
  	while(1) 										// Endlosschleife
  		{	



			char vdip1_ok = 0x00;
			char batch 	  = 0x00;

			vdip1_ok = vdip1_pruefen (vdip1_check);		// VDIP1 Zustand abfragen "2" wird bergeben 


    		if(vdip1_ok == vdip1_Programmier_Port_ok)	// Prfung auf Programmier Port Zustand
				{ 										// VDIP1 Zustand OK und kann jetzt neue Kommandos annehmen

      				switch (batch) 									// Batchprozess
						{ 
        					case 0: vdip1_cmd_schreiben("ECS"); 	// Erweiterte Kommando Set einstellen
                					break;	

							case 1: vdip1_cmd_schreiben("IPA"); 	// Alle Zeichen werden als ASCII Zeichen gesendet
                					break;

							case 2: vdip1_cmd_schreiben("FWV"); 	// FWV Firmwareversion anzeigen
                					break;

							case 3: vdip1_cmd_schreiben("DIR");		// Anzeige des USB_Stick inhaltes
                					break;

							case 4: vdip1_txtfile_schreiben("VNCFILE1.TXT",file1[0],file_neu);    // Textfile auf USB-Stick erzeugen
                					break;

							case 5: vdip1_txtfile_schreiben("VNCFILE1.TXT",file1[0],file_erweitern); // gleiches Textfile auf USB-Stick anhngen
                					break;

					//		case 6: vdip1_bin_file_bescheiben("FILE2.BIN", file2, 80);	 // Binrfile auf USB-Stick erzeugen
                	//				break;
							
        
					//		case 7: vnc_rd_file("VNCFILE1.TXT",test_buffer,20);          // File (Teil) von USB-Stick in Speicher einlesen
                	//				break;
      					}

	  				_delay_ms(1000); 				// Pause
      				batch++;    
					    							// nchster Schritt
     				while (batch ==8)
					//if (batch == 8) 
						{

							batch = 3; 				// Funktionen wiederholen fr Testzwecke			 
						}
    			}
  		}
	return 0;
	}

//************************************************************************
// Funktionen (Unterprogramme)
//************************************************************************


// ***********************************************************************
// VDIP1 - USB HOST CONTROLLER - SPI-MODE (C: Master, VDIP1: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.66 getestet
// ***********************************************************************

// ***********************************************************************
// VDIP1 - Intialisierung SPI-Interface fr VDIP1 ber SPI Mode
// ***********************************************************************
void vdip1_init(void) 
	{
		R_CS_VDIP;    								// SPI-Interface inaktiv
		R_SCLK;    									// Ruhezustand der SPI-Leitungen herstellen
		S_SDI;										// RW-Bit (0=Write)
		R_RE_VDIP;    								// Hardware-Reset VNC1L
		_delay_ms(100); 							// Pause
		S_RE_VDIP;		    						// VNC1L starten
	}

// ***********************************************************************
// VDIP1 1Byte auslesen

// bergabe: lese_funktion 0=Datenbyte, 1=Statusbyte
// Rckgabe: 	Datenbyte oder Statusbit ber die Globale VDIP1_Status Variable
// ***********************************************************************
void vdip1_byte_lesen(char lese_funktion) 
	{
		char daten;									// Daten definieren
		unsigned char stelle;						// Stelle definieren


		daten   = 0;									// definierter Start
		stelle  = 0;									// definierter Start
//------------------------------------------------------------------------
		S_CS_VDIP;									// SPI-Interface aktiv
		S_SDI;           							// Startbit
		S_SCLK; 										// L/H/L Flanke
		R_SCLK; 										// L/H/L Flanke
		S_SDI;           							// RW-Bit (1=Read)
		S_SCLK; 										// L/H/L Flanke
		R_SCLK; 										// L/H/L Flanke

  		if (lese_funktion)
  			{ 
  				S_SDI;								// wenn lese Funkton !=0 dann Statusbyte lesen
			} 
  		else
  			{ 
  				R_SDI; 								// ansonsten Daten lesen	
			}

		S_SCLK; 										// L/H/L Flanke
		R_SCLK;		 									// L/H/L Flanke
//------------------------------------------------------------------------
 
		for ( stelle = 0x80; stelle > 0; stelle >>= 1 )	// 8 Datenbits empfangen (MSB zuerst)
			{
    			if (SDO_VDIP) 						// wenn SD0 == 1 dann
					{
						daten |= stelle;			// verodere Daten mit Stelle (Bitweises schreiben)
    				}								

				S_SCLK; 								// L/H/L Flanke
				R_SCLK; 								// L/H/L Flanke		
  			}
//------------------------------------------------------------------------
  // Statusbit einlesen
  	vdip1_status = SDO_VDIP;     					// Statusbit (0=Neue Daten, 1=alte Daten)

	S_SCLK; 											// L/H/L Flanke
	R_SCLK; 											// L/H/L Flanke
//------------------------------------------------------------------------
  // End Sequenz
	R_CS_VDIP;           							// SPI-Interface inaktiv
	S_SCLK; 											// L/H/L Flanke
	R_SCLK;												// L/H/L Flanke

	if (vdip1_status==0)							// empfangenes Zeichen ist ein neues Zeichen
		{
    		for ( stelle = 9; stelle; stelle-- ) 	// Empfangsbuffer um eine Stelle verschieben
				{
					lese_buffer[stelle] = lese_buffer[stelle - 1];
				}

			lese_buffer[0] = daten; 				// neues Zeichen an 1. Stelle speichern
		}
		
	PORTA = lese_buffer[0];
//	_delay_ms(5000);
//	PORTA = 0xFF;
//	_delay_ms(500);
	} 

//************************************************************************
// von VDIP1 Auf Antwortstring warten und analysieren

// bergabe: 1 = Prfung auf Programmier-Port und Fehlermeldungen
//           2 = Prfung nur auf Programmier-Port 					//***** zur Zeit ausgewhlt
// Rckgabe: 1 = Programmier-Port empfangen
//           2 = Kommando File empfangen
//************************************************************************
char vdip1_pruefen(char buffer_check)								//*********************************************************
	{																//hier kann CODE gekzt werden case1 wird nicht gebraucht
		char antw = 0;											//*********************************************************

		while(antw = 1) 
			{   			
/*Probe*/		vdip1_cmd_schreiben("ECS");
				vdip1_byte_lesen(0); 								// Ein Zeichen per SPI von VNC1L lesen (0) == Daten lesen
	
				switch (buffer_check) 								// Kommando File
					{
        				case 1: if ((lese_buffer[2] == 'l') && (lese_buffer[1] == 'e') && (lese_buffer[0] == 'd')) 
										{
											antw = 2;						
																	// kein break; hier!!!
                						}
			
        				case 2: if ((lese_buffer[2] == 0x5C) && (lese_buffer[1] == '>') && (lese_buffer[0] == 0x0D)) // Test auf \> und CR war ok							// nur Prompt Check
									{
                  						antw = 1;						// USB Stick erkannt Freigabe erteilt
                  						lese_buffer[0] = 0; 			// lese Buffer letzes Zeichen lschen

								break;
    				}
    	
				if (antw == 2)
					{

						vdip1_byte_schreiben(0x0D); 				// zustzliches CR um Fehler zu besttigen, 
																	// dann kommt wieder ein Programmier Port Test
					}
			}
  	return antw;
	}

// ***********************************************************************
// VNC1L - Senden eines Zeichens im SPI Mode

// bergabe: daten:		Auszugebendes Zeichen
// Rckgabe: statusbit:	0= OK 	
//						1= nicht OK
// ***********************************************************************
char vdip1_byte_schreiben(char daten) 
	{
		char stelle;
		char status;
//------------------------------------------------------------------------
		S_CS_VDIP;									// SPI-Interface aktiv
		S_SDI;           							// Startbit (immer 1)
		S_SCLK; 										// L/H/L Flanke
		R_SCLK;											// L/H/L Flanke
		R_SDI;           							// RW-Bit (0=Write)
		S_SCLK; 										// L/H/L Flanke
		R_SCLK; 										// L/H/L Flanke
		R_SDI;           							// ADDR (0=Daten)
		S_SCLK; 										// L/H/L Flanke
		R_SCLK;											// L/H/L Flanke
 //------------------------------------------------------------------------
  // Datenbyte senden
  		for ( stelle = 0x80; stelle; stelle >>= 1 ) 	// 8 Datenbits senden (MSB zuerst)
			{
    			if (daten & stelle)					// Datenbit bertragen		
    				{
						S_SDI;
					}
				else
					{
						R_SDI;
					}	
				S_SCLK;									// L/H/L Flanke
				R_SCLK;									// L/H/L Flanke

  			}	
 //------------------------------------------------------------------------
  // Statusbit einlesen
		status = SDO_VDIP;							// Statusbit (0=OK, 1=nicht OK)
		S_SCLK;											// L/H/L Flanke
		R_SCLK;											// L/H/L Flanke
  //------------------------------------------------------------------------
  // End Sequenz
  		R_CS_VDIP;									// SPI-Interface inaktiv
		S_SCLK;											// L/H/L Flanke
		R_SCLK;											// L/H/L Flanke

	return (status);
	} 

// ***********************************************************************
// VDIP1 - Senden eines Kommandos
// bergabe: schreibe_cmd : Kommando (Adresse)
// ***********************************************************************
void vdip1_cmd_schreiben(char *schreibe_cmd) 
	{
		unsigned long i;
  		
		for(i = strlen(*schreibe_cmd); i; i--)		// Stringlnge an schleife bergeben
			{
    			vdip1_byte_schreiben(*schreibe_cmd);// Kommandobefehl ins VDIP1 schreiben
				schreibe_cmd ++;					// Kommandozeiger ++
  			}

  		vdip1_byte_schreiben(0x0D);					// Ende Zeichen
  		_delay_ms(200); 							// Pause, um VDIP1 Zeit zur Antwort zu geben
	}

// ***********************************************************************
// VNC1L - Erzeugen eines Binr-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Datenblockes
//           groee		: Anzahl der Bytes
// ***********************************************************************
void vdip1_bin_file_bescheiben(char *filename, char *fileinhalt, int groesse)
	{
		int i;
		char batch = 0;
 
  		do 
			{
    			switch (batch) 													// Batchprozess
					{ 
      					case 0: sprintf (&sende_buffer[0],"OPW %s",filename);	// File ffnen bzw. Erstellen
               					vdip1_cmd_schreiben(sende_buffer);
 	              				break;
      
	  					case 1:	vdip1_cmd_schreiben("SEK 0");  					// File Position immer auf Anfang setzen 
								break;											// (knnte man auch auf jede Pos. setzen!)
    	           			
	  					case 2: sprintf (&sende_buffer[0],"WRF %d",groesse);	// File Byteanzahl
               					vdip1_cmd_schreiben(sende_buffer);
               				
								for(i = 0; i < groesse; i++) 
									{
                 						vdip1_byte_schreiben(*fileinhalt);
                 						fileinhalt++;
               						}
               					break;
      
	  					case 3: sprintf (&sende_buffer[0],"CLF %s",filename);	// File schlieen
               					vdip1_cmd_schreiben(sende_buffer);
               					break;
   					}
	
				_delay_ms(500); 												// Pause
		    	batch++;    													// nchster Schritt
    			
				if (batch == 4) 
					{
						return; 												// Fertig
  					}

  			} 
		while (vdip1_pruefen(vdip1_ch) == vdip1_Programmier_Port_ok); 			// Prfung auf VDIP1 Zustand
	}

// ***********************************************************************
// VNC1L - Erzeugen eines Text-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Textblockes
//           funktion   : File 0=Neu oder 1=Inhalt anhngen
// ***********************************************************************
void vdip1_txtfile_schreiben(char *filename, char *fileinhalt, char funktion) 
	{
		int  i;
		int  file_laenge;
		int  file_null;
		char *datenadresse;
		char batch   = 0;

		datenadresse = fileinhalt;			
  		file_laenge = 0;
  		file_null   = 0;
  		
		while (*datenadresse != 0xFF) 
			{
    			if (*datenadresse) 
					{
						file_laenge++;
					}
				 else 
				 	{
						file_null++;
					}

    			datenadresse++;
  			}

  		do 
			{
	   			switch (batch) 													// Batchprozess
					{ 
      					case 0: sprintf (&sende_buffer[0],"OPW %s",filename);	// File ffnen bzw. Erstellen
               					vdip1_cmd_schreiben(sende_buffer);
               					break;
      
	  					case 1:  if (!funktion)									// File Position bei Bedarf auf Anfang setzen
									{
    	             					vdip1_cmd_schreiben("SEK 0");
										break;
            	   					}
								else 
									{
										batch++;  								// Nun wird sofort auch noch case 2: ausgefhrt
      								}
									
						case 2: sprintf (&sende_buffer[0],"WRF %d",file_laenge);// File Byteanzahl
               					vdip1_cmd_schreiben(sende_buffer);
               					datenadresse = fileinhalt;
               						
								for(i = 0; i < (file_laenge + file_null); i++) 
									{
                 						if (*datenadresse) 						// NULL-Terminierung ausblenden
											{	
												vdip1_byte_schreiben(*datenadresse); 		
                 							}

										datenadresse++;
               						}
               					break;
      
	  					case 3: sprintf (&sende_buffer[0],"CLF %s",filename);	// File schlieen
           						vdip1_cmd_schreiben(sende_buffer);
           						break;
    				} 															// Switch Klammer

				_delay_ms(500); 												// Pause
    			batch++;    													// nchster Schritt
    			
				if (batch == 4) 
					{
						return; 												// Fertig
  					}
  			} 																	// do Klammer
  		while (vdip1_pruefen(vdip1_ch) == vdip1_Programmier_Port_ok);			// Prfung auf nchstes Promptzeichen >
	}
